@@ -447,7 +447,7 @@ wzd_user_t * FCN_GET_USER(uid_t uid)
       out_log(PGSQL_LOG_CHANNEL,"PGsql: too many groups for user %s, dropping others\n",user->username);
       break;
     }
-    if (wzd_row_get_uint(&j, res, 0 /* query asks only one column */)==0)
+    if (wzd_row_get_uint_offset(&j, res, i, 0 /* query asks only one column */)==0)
       user->groups[i] = j;
   }
   user->group_num = i;
@@ -549,7 +549,7 @@ wzd_group_t * FCN_GET_GROUP(gid_t gid)
   free(query);
 
   for (index=0; index<PQntuples(res); index++) {
-    wzd_row_get_string(group->ip_allowed[index], MAX_IP_LENGTH, res, 0 /* query asks only one column */);
+    wzd_row_get_string_offset(group->ip_allowed[index], MAX_IP_LENGTH, res, index, 0 /* query asks only one column */);
   }
 
   PQclear(res);
