@@ -226,6 +226,7 @@ int config_has_key(wzd_configfile_t * file, const char * groupname, const char *
 
 /** \brief Returns the value associated with \a key under \a groupname.
  * \return the value, or NULL if the key is not found
+ * The returned value is a pointer to the object, it must not be freed.
  */
 char * config_get_value(const wzd_configfile_t * file, const char * groupname, const char * key)
 {
@@ -398,8 +399,8 @@ int config_get_boolean(wzd_configfile_t * file, const char * groupname, const ch
     return -1;
   }
 
-  if (strcmp(value,"true")==0 || strcmp(value,"1")==0) return 1;
-  if (strcmp(value,"false")==0 || strcmp(value,"0")==0) return 0;
+  if (strcmp(value,"true")==0  || strcmp(value,"1")==0 || strcmp(value,"yes")==0) return 1;
+  if (strcmp(value,"false")==0 || strcmp(value,"0")==0 || strcmp(value,"no")==0 ) return 0;
 
   if (errcode) *errcode = CF_ERROR_PARSE;
 
