@@ -77,7 +77,6 @@ static void _cfg_parse_modules(const wzd_configfile_t * file, wzd_config_t * con
 static void _cfg_parse_permissions(const wzd_configfile_t * file, wzd_config_t * config);
 static void _cfg_parse_pre_ip(const wzd_configfile_t * file, wzd_config_t * config);
 static void _cfg_parse_sections(const wzd_configfile_t * file, wzd_config_t * config);
-static void _cfg_parse_sitefiles(const wzd_configfile_t * file, wzd_config_t * config);
 static void _cfg_parse_vfs(const wzd_configfile_t * file, wzd_config_t * config);
 
 
@@ -136,33 +135,21 @@ void cfg_free(wzd_config_t * cfg)
 {
   WZD_ASSERT_VOID(cfg != NULL);
 
-  if (cfg->htab) chtbl_destroy((CHTBL*)cfg->htab);
-  wzd_free(cfg->htab);
-
   wzd_free(cfg->logfilename);
   wzd_free(cfg->config_filename);
   wzd_free(cfg->pid_file);
   wzd_free(cfg->dir_message);
   wzd_free(cfg->xferlog_name);
   wzd_free(cfg->logdir);
-  wzd_free(cfg->backend.name);
-
-  wzd_free(mainConfig->site_config.file_ginfo);
-  wzd_free(mainConfig->site_config.file_group);
-  wzd_free(mainConfig->site_config.file_groups);
-  wzd_free(mainConfig->site_config.file_help);
-  wzd_free(mainConfig->site_config.file_rules);
-  wzd_free(mainConfig->site_config.file_swho);
-  wzd_free(mainConfig->site_config.file_user);
-  wzd_free(mainConfig->site_config.file_users);
-  wzd_free(mainConfig->site_config.file_vfs);
-  wzd_free(mainConfig->site_config.file_who);
+  wzd_free(cfg->backend.filename);
 
   event_mgr_free(cfg->event_mgr);
   wzd_free(cfg->event_mgr);
 
   commands_fini(cfg->commands_list);
 
+  ip_list_free(cfg->login_pre_ip_checks);
+
   config_free(cfg->cfg_file);
 
   memset(cfg, 0, sizeof(wzd_config_t));
@@ -178,7 +165,7 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
   wzd_string_t * str, * ptr;
   char * p;
   unsigned long ul;
-  int ret;
+  int ret, err;
   int i;
 
   cfg = wzd_malloc(sizeof(wzd_config_t));
@@ -187,6 +174,15 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
   cfg_init(cfg);
   cfg->cfg_file = file;
 
+  /* CHECK_IP_BEFORE_LOGIN */
+  str = config_get_string(file, "GLOBAL", "check_ip_before_login", NULL);
+  if (str) {
+    if (strcasecmp(str_tochar(str),"allow")==0 || strcmp(str_tochar(str),"1")==0) {
+      CFG_SET_OPTION(cfg,CFG_OPT_CHECKIP_LOGIN);
+    }
+    str_deallocate(str);
+  }
+
   /* DATA_BUFFER_LENGTH */
   str = config_get_string(file, "GLOBAL", "data_buffer_length", NULL);
   if (str) {
@@ -199,13 +195,9 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
   }
 
   /* DENY_ACCESS_FILES_UPLOADED */
-  str = config_get_string(file, "GLOBAL", "deny_access_files_uploaded", NULL);
-  if (str) {
-    if (strcasecmp(str_tochar(str),"allow")==0 || strcmp(str_tochar(str),"1")==0) {
-      CFG_SET_OPTION(cfg,CFG_OPT_DENY_ACCESS_FILES_UPLOADED);
-    }
-    str_deallocate(str);
-  }
+  ret = config_get_boolean(file, "GLOBAL", "deny_access_files_uploaded", &err);
+  if (err == CF_OK && (ret))
+    CFG_SET_OPTION(cfg,CFG_OPT_DENY_ACCESS_FILES_UPLOADED);
 
   /* DIR_MESSAGE */
   str = config_get_string(file, "GLOBAL", "dir_message", NULL);
@@ -215,22 +207,14 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
   }
 
   /* DISABLE_IDENT */
-  str = config_get_string(file, "GLOBAL", "disable_ident", NULL);
-  if (str) {
-    if (strcasecmp(str_tochar(str),"allow")==0 || strcmp(str_tochar(str),"1")==0) {
-      CFG_SET_OPTION(cfg,CFG_OPT_DISABLE_IDENT);
-    }
-    str_deallocate(str);
-  }
+  ret = config_get_boolean(file, "GLOBAL", "disable_ident", &err);
+  if (err == CF_OK && (ret))
+    CFG_SET_OPTION(cfg,CFG_OPT_DISABLE_IDENT);
 
   /* DISABLE_TLS */
-  str = config_get_string(file, "GLOBAL", "disable_tls", NULL);
-  if (str) {
-    if (strcasecmp(str_tochar(str),"allow")==0 || strcmp(str_tochar(str),"1")==0) {
-      CFG_SET_OPTION(cfg,CFG_OPT_DISABLE_TLS);
-    }
-    str_deallocate(str);
-  }
+  ret = config_get_boolean(file, "GLOBAL", "disable_tls", &err);
+  if (err == CF_OK && (ret))
+    CFG_SET_OPTION(cfg,CFG_OPT_DISABLE_TLS);
 
   /* DYNAMIC_IP */
   str = config_get_string(file, "GLOBAL", "dynamic_ip", NULL);
@@ -240,13 +224,9 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
   }
 
   /* HIDE_DOTTED_FILES */
-  str = config_get_string(file, "GLOBAL", "hide_dotted_files", NULL);
-  if (str) {
-    if (strcasecmp(str_tochar(str),"allow")==0 || strcmp(str_tochar(str),"1")==0) {
-      CFG_SET_OPTION(cfg,CFG_OPT_HIDE_DOTTED_FILES);
-    }
-    str_deallocate(str);
-  }
+  ret = config_get_boolean(file, "GLOBAL", "hide_dotted_files", &err);
+  if (err == CF_OK && (ret))
+    CFG_SET_OPTION(cfg,CFG_OPT_HIDE_DOTTED_FILES);
 
   /* IP XXX to be implemented */
 
@@ -255,10 +235,6 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
   if (str) {
     cfg->logfilename = strdup(str_tochar(str));
     str_deallocate(str);
-  } else {
-    out_err(LEVEL_CRITICAL,"No logfile found !\n");
-    cfg_free(cfg);
-    return NULL;
   }
 
   /* LOGLEVEL */
@@ -267,6 +243,7 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
     i = str2loglevel(str_tochar(str));
     if( i==-1 ) {
       out_err(LEVEL_HIGH,"valid levels are lowest, flood, info, normal, high, critical\n");
+      cfg->cfg_file = NULL;
       cfg_free(cfg);
       return NULL;
     }
@@ -348,13 +325,22 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
     ul = strtoul(str_tochar(str),NULL,0);
     if (ul > 65535) {
       out_err(LEVEL_CRITICAL,"Invalid port number !\n");
+      cfg->cfg_file = NULL;
       cfg_free(cfg);
       return NULL;
     }
     cfg->port = ul;
     str_deallocate(str);
+  } else {
+    /* set default value to FTP port */
+    config_set_value(file, "GLOBAL", "port", "21");
   }
 
+  /* REJECT_UNKNOWN_USERS */
+  ret = config_get_boolean(file, "GLOBAL", "reject_unknown_users", &err);
+  if (err == CF_OK && (ret))
+    CFG_SET_OPTION(cfg,CFG_OPT_REJECT_UNKNOWN_USERS);
+
   /* SERVER_GID */
   str = config_get_string(file, "GLOBAL", "server_gid", NULL);
   if (str) {
@@ -429,19 +415,9 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
   }
 
   /* USE SYSLOG */
-  str = config_get_string(file, "GLOBAL", "use_syslog", NULL);
-  if (str) {
-    if (strcasecmp(str_tochar(str),"deny")==0 || strcmp(str_tochar(str),"0")==0) {
-      CFG_CLR_OPTION(cfg,CFG_OPT_USE_SYSLOG);
-    }
-    else if (strcasecmp(str_tochar(str),"allow")==0 || strcmp(str_tochar(str),"1")==0) {
-      CFG_SET_OPTION(cfg,CFG_OPT_USE_SYSLOG);
-    }
-    else {
-      out_log(LEVEL_HIGH,"Invalid value for option use_syslog\n");
-    }
-    str_deallocate(str);
-  }
+  ret = config_get_boolean(file, "GLOBAL", "use_syslog", &err);
+  if (err == CF_OK && (ret))
+    CFG_SET_OPTION(cfg,CFG_OPT_USE_SYSLOG);
 
   /* XFERLOG */
   str = config_get_string(file, "GLOBAL", "xferlog", NULL);
@@ -460,7 +436,7 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
       if (!ret) {
         if (cfg->backend.handle == NULL) {
           /*        i = backend_init(value);*/
-          cfg->backend.name = wzd_strdup(str_tochar(ptr));
+          cfg->backend.filename = wzd_strdup(str_tochar(ptr));
         } else { /* multiple backends ?? */
           ret=0;
         }
@@ -473,8 +449,6 @@ wzd_config_t * cfg_store(wzd_configfile_t * file, int * error)
 
   _cfg_parse_pre_ip(file, cfg);
 
-  _cfg_parse_sitefiles(file, cfg);
-
   _cfg_parse_messages(file, cfg);
   _cfg_parse_sections(file, cfg);
   _cfg_parse_vfs(file, cfg);
@@ -498,7 +472,7 @@ static void _cfg_parse_pre_ip(const wzd_configfile_t * file, wzd_config_t * conf
   int i, value;
   int err;
   char * address, * check;
-  
+
   array = config_get_keys(file,"pre_ip_check",&err);
   if (!array) return;
 
@@ -534,7 +508,7 @@ static void _cfg_parse_crontab(const wzd_configfile_t * file, wzd_config_t * con
   char * cron_name;
   wzd_string_t * cron_value;
   wzd_string_t * min, * hour, * day, * month, * day_of_week;
-  
+
   array = config_get_keys(file,"cron",&err);
   if (!array) return;
 
@@ -577,7 +551,7 @@ static void _cfg_parse_custom_commands(const wzd_configfile_t * file, wzd_config
   int err;
   char * command_name;
   wzd_string_t * value;
-  
+
   array = config_get_keys(file,"custom_commands",&err);
   if (!array) return;
 
@@ -619,7 +593,7 @@ static void _cfg_parse_events(const wzd_configfile_t * file, wzd_config_t * conf
   char * event_name;
   wzd_string_t * event, * value;
   unsigned long eventmask;
-  
+
   array = config_get_keys(file,"events",&err);
   if (!array) return;
 
@@ -641,6 +615,7 @@ static void _cfg_parse_events(const wzd_configfile_t * file, wzd_config_t * conf
         } else {
           out_log(LEVEL_HIGH,"ERROR while adding event: %s\n",event_name);
         }
+        str_deallocate(command);
       }
     } else {
       out_log(LEVEL_HIGH,"ERROR incorrect syntax for event: %s\n",event_name);
@@ -662,7 +637,7 @@ static void _cfg_parse_messages(const wzd_configfile_t * file, wzd_config_t * co
   wzd_string_t * value;
   char * message;
   unsigned long ul;
-  
+
   array = config_get_keys(file,"messages",&err);
   if (!array) return;
 
@@ -693,7 +668,7 @@ static void _cfg_parse_modules(const wzd_configfile_t * file, wzd_config_t * con
   int err;
   char * module_name;
   wzd_string_t * permission;
-  
+
   array = config_get_keys(file,"modules",&err);
   if (!array) return;
 
@@ -730,7 +705,7 @@ static void _cfg_parse_permissions(const wzd_configfile_t * file, wzd_config_t *
   int err;
   char * permission_name;
   wzd_string_t * permission;
-  
+
   array = config_get_keys(file,"perms",&err);
   if (!array) return;
 
@@ -759,7 +734,7 @@ static void _cfg_parse_sections(const wzd_configfile_t * file, wzd_config_t * co
   int err;
   char * section_name;
   wzd_string_t * section, * section_mask;
-  
+
   array = config_get_keys(file,"sections",&err);
   if (!array) return;
 
@@ -784,47 +759,6 @@ static void _cfg_parse_sections(const wzd_configfile_t * file, wzd_config_t * co
   str_deallocate_array(array);
 }
 
-static void _cfg_parse_sitefiles(const wzd_configfile_t * file, wzd_config_t * config)
-{
-  wzd_string_t * filename;
-  
-  filename = config_get_string(file, "GLOBAL", "sitefile_ginfo", NULL);
-  if (filename) { config->site_config.file_ginfo = wzd_strdup(str_tochar(filename)); }
-  str_deallocate(filename);
-  
-  filename = config_get_string(file, "GLOBAL", "sitefile_group", NULL);
-  if (filename) { config->site_config.file_group = wzd_strdup(str_tochar(filename)); }
-  str_deallocate(filename);
-  
-  filename = config_get_string(file, "GLOBAL", "sitefile_groups", NULL);
-  if (filename) { config->site_config.file_groups = wzd_strdup(str_tochar(filename)); }
-  str_deallocate(filename);
-  
-  filename = config_get_string(file, "GLOBAL", "sitefile_help", NULL);
-  if (filename) { config->site_config.file_help = wzd_strdup(str_tochar(filename)); }
-  str_deallocate(filename);
-  
-  filename = config_get_string(file, "GLOBAL", "sitefile_swho", NULL);
-  if (filename) { config->site_config.file_swho = wzd_strdup(str_tochar(filename)); }
-  str_deallocate(filename);
-  
-  filename = config_get_string(file, "GLOBAL", "sitefile_user", NULL);
-  if (filename) { config->site_config.file_user = wzd_strdup(str_tochar(filename)); }
-  str_deallocate(filename);
-
-  filename = config_get_string(file, "GLOBAL", "sitefile_users", NULL);
-  if (filename) { config->site_config.file_users = wzd_strdup(str_tochar(filename)); }
-  str_deallocate(filename);
-
-  filename = config_get_string(file, "GLOBAL", "sitefile_who", NULL);
-  if (filename) { config->site_config.file_who = wzd_strdup(str_tochar(filename)); }
-  str_deallocate(filename);
-
-  filename = config_get_string(file, "GLOBAL", "sitefile_vfs", NULL);
-  if (filename) { config->site_config.file_vfs = wzd_strdup(str_tochar(filename)); }
-  str_deallocate(filename);
-}
-
 static void _cfg_parse_vfs(const wzd_configfile_t * file, wzd_config_t * config)
 {
   wzd_string_t ** array;
@@ -833,7 +767,7 @@ static void _cfg_parse_vfs(const wzd_configfile_t * file, wzd_config_t * config)
   char * key_name;
   wzd_string_t * value, * physical_path, * virtual_path, * permissions;
   char delimiter[2];
-  
+
   array = config_get_keys(file,"vfs",&err);
   if (!array) return;
 
