@@ -128,6 +128,7 @@ typedef struct
 #include "wzd_fs.h"
 #include "wzd_cache.h"
 #include "wzd_perm.h"
+#include "wzd_vfs.h"
 
 
 
@@ -1428,13 +1429,14 @@ int file_chown(const char *filename, const char *username, const char *groupname
 int file_mkdir(const char *dirname, unsigned int mode, wzd_context_t * context)
 {
   int ret;
+  int err;
   wzd_user_t * user;
 
   user = GetUserByID(context->userid);
 
   ret = _checkPerm(dirname,RIGHT_MKDIR,user);
   if (ret) return E_NOPERM;
-  ret = fs_mkdir(dirname,0755);
+  ret = fs_mkdir(dirname,0755,&err);
 
   return (ret) ? E_COMMAND_FAILED : E_OK;
 }
@@ -1789,6 +1791,32 @@ struct wzd_file_t * file_stat(const char *filename, wzd_context_t * context)
 /*  if (fs_stat(filename,&s))
     return NULL;*/
 
+  /* check for VFS */
+  {
+    wzd_vfs_t * vfs = mainConfig->vfs;
+    char * buffer_vfs;
+
+    while (vfs) {
+      buffer_vfs = vfs_replace_cookies(vfs->virtual_dir,context);
+      if (!buffer_vfs) {
+        out_log(LEVEL_CRITICAL,"vfs_replace_cookies returned NULL for %s\n",vfs->virtual_dir);
+        vfs = vfs->next_vfs;
+        continue;
+      }
+
+      if (DIRCMP(buffer_vfs,filename)==0) {
+        /* ok, we have a candidate */
+        file = file_stat(vfs->physical_dir,context);
+        wzd_free(buffer_vfs);
+        return file;
+      }
+
+      wzd_free(buffer_vfs);
+      vfs = vfs->next_vfs;
+    }
+
+  }
+
   file = NULL;
 
   /* find the dir containing the perms file */
@@ -1820,7 +1848,7 @@ struct wzd_file_t * file_stat(const char *filename, wzd_context_t * context)
         return NULL;
       }
     }
-  } /* ! isdir */
+  } /* ! exists */
 
 
 /*  strcpy(stripped_filename,ptr+1);*/
