@@ -69,6 +69,7 @@
 #include "wzd_vars.h"
 #include "wzd_vfs.h"
 #include "wzd_cache.h"
+#include "wzd_configfile.h"
 #include "wzd_events.h"
 #include "wzd_file.h"
 #include "wzd_fs.h"
@@ -641,7 +642,7 @@ int do_site_chpass(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   str_deallocate(new_pass);
 
   /* commit to backend */
-  ret = backend_mod_user(mainConfig->backend.name,user->username,user,mod_type);
+  ret = backend_mod_user(mainConfig->backend.filename,user->username,user,mod_type);
 
   if (ret)
     ret = send_message_with_args(501,context,"An error occurred during password change");
@@ -1392,16 +1393,77 @@ int do_site_rusage(wzd_string_t * ignored, wzd_string_t *param, wzd_context_t *
 /********************* do_site_savecfg *********************/
 int do_site_savecfg(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_t * context)
 {
-  send_message_with_args(501,context,"Not yet implemented");
-  return 1;
+  wzd_string_t * data = NULL;
+  size_t length = 0, written;
+  int fd;
+  int do_backup;
+  int err = 0;
 
-#if 0
-  if( wzd_savecfg() )
+  do_backup = config_get_boolean(mainConfig->cfg_file, "GLOBAL", "backup config", &err);
+  if (err == CF_ERROR_NOT_FOUND) do_backup = 0; /* default: no backup */
+  else if (err != CF_OK) {
+    out_log(LEVEL_HIGH,"ERROR Could not save config (error while getting option 'backup config')\n");
     send_message_with_args(501,context,"Cannot save server config");
-  else
-    send_message_with_args(200,context,"Server config saved");
+    return -1;
+  }
+
+  if (do_backup)
+  {
+    int backup_fd;
+    char buffer[1024];
+    ssize_t ret;
+    char * filename = NULL;
+    size_t name_length;
+
+    name_length = strlen(mainConfig->config_filename);
+    filename = malloc(name_length + 6);
+    snprintf(filename,name_length+5,"%s.old",mainConfig->config_filename);
+    out_log(LEVEL_FLOOD,"DEBUG Config file saved to [%s]\n",filename);
+
+    fd = open(mainConfig->config_filename, O_RDONLY, 0644);
+    backup_fd = open(filename, O_CREAT|O_WRONLY|O_TRUNC, 0644);
+    if (fd < 0 || backup_fd < 0) {
+      out_log(LEVEL_HIGH,"ERROR Could not save config (error when opening files)\n");
+      send_message_with_args(501,context,"Cannot save server config");
+      close(fd);
+      close(backup_fd);
+      return -1;
+    }
+    while ((ret = read(fd,buffer,sizeof(buffer))) > 0) {
+      write(backup_fd,buffer,ret);
+    }
+    close(fd);
+    close(backup_fd);
+  }
+
+  out_log(LEVEL_NORMAL,"INFO saving config to %s\n",mainConfig->config_filename);
+
+  fd = open(mainConfig->config_filename, O_WRONLY | O_TRUNC, 0644); /** XXX file mode is hardcoded */
+  if (fd < 0) {
+    out_log(LEVEL_HIGH,"ERROR Could not save config (error while creating a temporary file: %d: %s)\n",errno,strerror(errno));
+    send_message_with_args(501,context,"Cannot save server config");
+    return -1;
+  }
+
+  data = config_to_data(mainConfig->cfg_file,&length);
+
+  if (data == NULL) {
+    out_log(LEVEL_HIGH,"ERROR Could not save config (error in config_to_data)\n");
+    send_message_with_args(501,context,"Cannot save server config");
+    close(fd);
+    return -1;
+  }
+
+  written = write(fd, str_tochar(data), length);
+  if (written != length) {
+    out_log(LEVEL_HIGH,"ERROR Could not save config (written %d bytes instead of %d\n",written,length);
+  }
+  close(fd);
+
+  send_message_with_args(200,context,"Server config saved");
+
+  str_deallocate(data);
   return 0;
-#endif
 }
 
 /********************* do_site_sections ******************/
@@ -1471,25 +1533,46 @@ int do_site_unlock(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
 /** user username
  */
 
-void do_site_user(const char *command_line, wzd_context_t * context)
+int do_site_user(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_t * context)
 {
   const char * username;
   int ret;
   wzd_user_t user;
   int uid;
+  wzd_string_t * str;
+  wzd_user_t * me = NULL;
 
-  username = command_line;
+  if (context) me = GetUserByID(context->userid); /*get self*/
+
+  username = str_tochar(command_line);
   if (!username) {
     do_site_help("user",context);
-    return;
+    return 0;
   }
   /* check that username exists */
   if ( backend_find_user(username,&user,&uid) ) {
     ret = send_message_with_args(501,context,"User does not exists");
-    return;
+    return 0;
+  }
+  if ( strchr(user.flags,FLAG_ULTRAHIDDEN )&&
+       (me) &&
+       (strcmp(username,me->username)!=0)/* do not hide to self ! */
+     ) {
+    /* for siteops we could send a different message, like 'user is hidden' */
+    ret = send_message_with_args(501,context,"User does not exists");
+    return 0;
   }
 
-  do_site_print_file(mainConfig->site_config.file_user,&user,NULL,context);
+  str = config_get_string(mainConfig->cfg_file,"GLOBAL","sitefile_user",NULL);
+  if (!str) {
+    ret = send_message_with_args(501,context,"File [GLOBAL] / sitefile_user does not exists");
+    return 0;
+  }
+
+  do_site_print_file(str_tochar(str),&user,NULL,context);
+
+  str_deallocate(str);
+  return 0;
 }
 
 /********************* do_site_utime ***********************/
@@ -1834,17 +1917,6 @@ int do_site_version(wzd_string_t * ignored, wzd_string_t * command_line, wzd_con
   return 0;
 }
 
-/********************* do_site_vfsls ***********************/
-
-/* XXX : just send last vfs */
-
-int do_site_vfsls(wzd_string_t * ignored, wzd_string_t * command_line, wzd_context_t * context)
-{
-  do_site_print_file(mainConfig->site_config.file_vfs,NULL,NULL,context);
-
-  return 0;
-}
-
 /********************* do_site_vfsadd **********************/
 /** vfsadd |/home/vfsroot|/physical/path| +O =user
  */
@@ -2127,43 +2199,12 @@ int do_site(wzd_string_t *command, wzd_string_t *command_line, wzd_context_t * c
     ret = send_message_with_args(250,context,"SITE: ","server is now closed");
     return 0;
   } else
-/******************* GINFO **********************/
-  if (strcmp(s_token,"site_groups")==0) {
-    do_site_print_file(mainConfig->site_config.file_groups,NULL,NULL,context);
-    return 0;
-  } else
-/******************* HELP ***********************/
-  if (strcmp(s_token,"site_help")==0) {
-    /* TODO check if there are arguments, and call specific help */
-    do_site_print_file(mainConfig->site_config.file_help,GetUserByID(context->userid),NULL,context);
-    return 0;
-  } else
 /******************** REOPEN ********************/
   if (strcmp(s_token,"site_reopen")==0) {
     mainConfig->site_closed = 0;
     ret = send_message_with_args(250,context,"SITE: ","server is now opened");
     return 0;
   } else
-/******************* SWHO ***********************/
-  if (strcmp(s_token,"site_swho")==0) {
-    do_site_print_file(mainConfig->site_config.file_swho,NULL,NULL,context);
-    return 0;
-  } else
-/******************* USER ***********************/
-  if (strcmp(s_token,"site_user")==0) {
-    do_site_user(str_tochar(command_line),context);
-    return 0;
-  } else
-/******************* USERS **********************/
-  if (strcmp(s_token,"site_users")==0) {
-    do_site_print_file(mainConfig->site_config.file_users,NULL,NULL,context);
-    return 0;
-  } else
-/******************* WHO ************************/
-  if (strcmp(s_token,"site_who")==0) {
-    do_site_print_file(mainConfig->site_config.file_who,NULL,NULL,context);
-    return 0;
-  } else
 /******************* UPTIME *********************/
   if (strcmp(s_token,"site_uptime")==0) {
     time_t t;
