@@ -52,6 +52,7 @@
 
 #include "wzd_structs.h"
 
+#include "wzd_configfile.h"
 #include "wzd_fs.h"
 #include "wzd_log.h"
 #include "wzd_misc.h"
@@ -145,7 +146,7 @@ int do_site_grpadd(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     newgroup.ip_allowed[i][0]='\0';
 
   /* add it to backend */
-  ret = backend_mod_group(mainConfig->backend.name,str_tochar(groupname),&newgroup,_GROUP_ALL);
+  ret = backend_mod_group(mainConfig->backend.filename,str_tochar(groupname),&newgroup,_GROUP_ALL);
 
   str_deallocate(groupname);
 
@@ -236,7 +237,7 @@ int do_site_grpdel(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   /* TODO XXX FIXME delete users belonging only to this group ? */
 
   /* commit changes to backend */
-  backend_mod_group(mainConfig->backend.name,str_tochar(groupname),NULL,_GROUP_ALL);
+  backend_mod_group(mainConfig->backend.filename,str_tochar(groupname),NULL,_GROUP_ALL);
   str_deallocate(groupname);
 
   ret = send_message_raw("200 Group deleted\r\n",context);
@@ -303,7 +304,7 @@ int do_site_grpren(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   str_deallocate(newgroupname);
 
   /* add it to backend */
-  ret = backend_mod_group(mainConfig->backend.name,oldgroup->groupname,&group,_GROUP_GROUPNAME);
+  ret = backend_mod_group(mainConfig->backend.filename,oldgroup->groupname,&group,_GROUP_GROUPNAME);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
@@ -315,6 +316,7 @@ int do_site_grpren(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
 
 int do_site_ginfo(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_t * context)
 {
+  wzd_string_t * str;
   wzd_string_t * groupname;
   int ret;
   wzd_group_t * group;
@@ -332,14 +334,23 @@ int do_site_ginfo(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
     return 0;
   }
 
+  str = config_get_string(mainConfig->cfg_file,"GLOBAL","sitefile_ginfo",NULL);
+  if (!str) {
+    ret = send_message_with_args(501,context,"File [GLOBAL] / sitefile_ginfo does not exists");
+    return 0;
+  }
+
   /* TODO XXX FIXME gadmins can see ginfo only on their primary group ? */
-  do_site_print_file(mainConfig->site_config.file_ginfo,NULL,group,context);
+  do_site_print_file(str_tochar(str),NULL,group,context);
+
+  str_deallocate(str);
 
   return 0;
 }
 
 int do_site_gsinfo(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_t * context)
 {
+  wzd_string_t * str;
   wzd_string_t * groupname;
   int ret;
   wzd_group_t * group;
@@ -357,7 +368,15 @@ int do_site_gsinfo(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     return 0;
   }
 
-  do_site_print_file(mainConfig->site_config.file_group,NULL,group,context);
+  str = config_get_string(mainConfig->cfg_file,"GLOBAL","sitefile_group",NULL);
+  if (!str) {
+    ret = send_message_with_args(501,context,"File [GLOBAL] / sitefile_group does not exists");
+    return 0;
+  }
+
+  do_site_print_file(str_tochar(str),NULL,group,context);
+
+  str_deallocate(str);
 
   return 0;
 }
@@ -440,7 +459,7 @@ int do_site_grpaddip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
   strncpy(group->ip_allowed[i],str_tochar(ip),MAX_IP_LENGTH-1);
 
   /* commit to backend */
-  backend_mod_group(mainConfig->backend.name,group->groupname,group,_GROUP_IP);
+  backend_mod_group(mainConfig->backend.filename,group->groupname,group,_GROUP_IP);
 
   ret = send_message_with_args(200,context,"Group ip added");
   str_deallocate(ip);
@@ -515,7 +534,7 @@ int do_site_grpdelip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
       return 0;
     }
     group->ip_allowed[ul][0] = '\0';
-    backend_mod_group(mainConfig->backend.name,group->groupname,group,_GROUP_IP);
+    backend_mod_group(mainConfig->backend.filename,group->groupname,group,_GROUP_IP);
     ret = send_message_with_args(200,context,"Group ip removed");
     return 0;
   } /* if (*ptr=='\0') */
@@ -528,7 +547,7 @@ int do_site_grpdelip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
       group->ip_allowed[i][0] = '\0';
       /* commit to backend */
       /* FIXME backend name hardcoded */
-      backend_mod_group(mainConfig->backend.name,group->groupname,group,_USER_IP);
+      backend_mod_group(mainConfig->backend.filename,group->groupname,group,_GROUP_IP);
       ret = send_message_with_args(200,context,"Group ip removed");
       str_deallocate(ip);
       return 0;
@@ -597,7 +616,7 @@ int do_site_grpratio(wzd_string_t *ignored, wzd_string_t *command_line, wzd_cont
   group->ratio = ratio;
 
   /* add it to backend */
-  ret = backend_mod_group(mainConfig->backend.name,group->groupname,group,_GROUP_RATIO);
+  ret = backend_mod_group(mainConfig->backend.filename,group->groupname,group,_GROUP_RATIO);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
@@ -716,7 +735,7 @@ int do_site_grpchange(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
   }
 
   /* find modification type */
-  mod_type = _USER_NOTHING;
+  mod_type = _GROUP_NOTHING;
 
   /* groupname */
   if (strcmp(str_tochar(field),"name")==0) {
@@ -790,7 +809,7 @@ int do_site_grpchange(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
   }
 
   /* commit to backend */
-  ret = backend_mod_group(mainConfig->backend.name,str_tochar(groupname),group,mod_type);
+  ret = backend_mod_group(mainConfig->backend.filename,str_tochar(groupname),group,mod_type);
 
   str_deallocate(groupname); str_deallocate(field); str_deallocate(value);
 
@@ -857,8 +876,9 @@ int do_site_group(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
     do_site_grpkill( cmd, command_line, context );
   } else if(strcmp( "change", str_tochar(cmd)) == 0) {
     do_site_grpchange( cmd, command_line, context );
-  } else if(strcmp( "list", str_tochar(cmd)) == 0) {
-    do_site_print_file(mainConfig->site_config.file_groups,NULL,NULL,context);
+    /** \todo FIXME implement SITE GROUP LIST */
+/*  } else if(strcmp( "list", str_tochar(cmd)) == 0) {
+    do_site_print_file(mainConfig->site_config.file_groups,NULL,NULL,context);*/
   } else {
     ret = send_message_with_args(501,context,"site group action invalid");
   }
