@@ -214,7 +214,7 @@ int do_site_adduser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   }
 
   /* add it to backend */
-  ret = backend_mod_user(mainConfig->backend.name,str_tochar(username),&user,_USER_ALL);
+  ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_ALL);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem adding user");
@@ -285,7 +285,7 @@ int do_site_deluser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   user->flags[length+1] = '\0';
 
   /* commit changes to backend */
-  backend_mod_user(mainConfig->backend.name,user->username,user,_USER_FLAGS);
+  backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_FLAGS);
 
   ret = send_message_with_args(200,context,"User deleted");
   return 0;
@@ -350,7 +350,7 @@ int do_site_readduser(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
   }
 
   /* commit changes to backend */
-  backend_mod_user(mainConfig->backend.name,user.username,&user,_USER_FLAGS);
+  backend_mod_user(mainConfig->backend.filename,user.username,&user,_USER_FLAGS);
 
   ret = send_message_with_args(200,context,"User undeleted");
   return 0;
@@ -402,7 +402,7 @@ int do_site_purgeuser(wzd_string_t *command_line, wzd_string_t *param, wzd_conte
     }
 
     /* commit changes to backend */
-    backend_mod_user(mainConfig->backend.name,str_tochar(username),NULL,_USER_ALL);
+    backend_mod_user(mainConfig->backend.filename,str_tochar(username),NULL,_USER_ALL);
     str_deallocate(username);
   } else { /* if (username) */
     /* TODO iterate users and purge those marked as deleted */
@@ -426,7 +426,7 @@ int do_site_purgeuser(wzd_string_t *command_line, wzd_string_t *param, wzd_conte
             }
           }
           /* commit changes to backend */
-          backend_mod_user(mainConfig->backend.name,user->username,NULL,_USER_ALL);
+          backend_mod_user(mainConfig->backend.filename,user->username,NULL,_USER_ALL);
         }
       }
       wzd_free (uid_list);
@@ -526,7 +526,7 @@ int do_site_addip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
   } while (ip);
 
   /* commit to backend */
-  backend_mod_user(mainConfig->backend.name,user->username,user,_USER_IP);
+  backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_IP);
 
   ret = send_message_with_args(200,context,"User ip(s) added");
   return 0;
@@ -630,7 +630,7 @@ int do_site_delip(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
   } while (ip);
 
   /* commit to backend */
-  backend_mod_user(mainConfig->backend.name,user->username,user,_USER_IP);
+  backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_IP);
   ret = send_message_with_args(200,context,"User ip(s) removed");
   return 0;
 }
@@ -661,12 +661,12 @@ int do_site_color(wzd_string_t *command_line, wzd_string_t *param, wzd_context_t
     *dst_ptr++ = FLAG_COLOR;
     *dst_ptr='\0';
     memcpy(me->flags,new_flags,MAX_FLAGS_NUM);
-    ret = backend_mod_user(mainConfig->backend.name,me->username,me,_USER_FLAGS);
+    ret = backend_mod_user(mainConfig->backend.filename,me->username,me,_USER_FLAGS);
     ret = send_message_with_args(200,context,"color mode ON");
   } else {
     *dst_ptr='\0';
     memcpy(me->flags,new_flags,MAX_FLAGS_NUM);
-    ret = backend_mod_user(mainConfig->backend.name,me->username,me,_USER_FLAGS);
+    ret = backend_mod_user(mainConfig->backend.filename,me->username,me,_USER_FLAGS);
     ret = send_message_with_args(200,context,"color mode OFF");
   }
   return 0;
@@ -711,6 +711,7 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   wzd_user_t * user, *me;
   unsigned int i;
   short is_gadmin;
+  char old_username[HARD_USERNAME_LENGTH];
 
   me = GetUserByID(context->userid);
   is_gadmin = (me->flags && strchr(me->flags,FLAG_GADMIN)) ? 1 : 0;
@@ -721,8 +722,9 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     return 0;
   }
   /* check if user  exists */
-  user = GetUserByName( str_tochar(username) );
+  strncpy(old_username,str_tochar(username),HARD_USERNAME_LENGTH);
   str_deallocate(username);
+  user = GetUserByName( old_username );
   if ( !user) {
     ret = send_message_with_args(501,context,"User does not exist");
     return 0;
@@ -799,7 +801,7 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
     if (newgroupid != (unsigned int)-1) {
       ret=0;
       for (i=0; i<user->group_num; i++)
-        if (newgroupid == user->groups[i]) { ret=1; break; } 
+        if (newgroupid == user->groups[i]) { ret=1; break; }
       if (ret) { /* remove from group, shift them */
         user->groups[i] = 0;
         for (;i<user->group_num-1; i++)
@@ -820,7 +822,7 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   /* perms */
   else if (strcmp(str_tochar(field),"perms")==0) {
     ul=strtoul(str_tochar(value),&ptr,0);
-    if (!*ptr) { mod_type = _USER_PERMS; user->userperms = ul;} 
+    if (!*ptr) { mod_type = _USER_PERMS; user->userperms = ul;}
   }
   /* flags */ /* TODO accept modifications style +f or -f */
   else if (strcmp(str_tochar(field),"flags")==0) {
@@ -921,7 +923,7 @@ int do_site_change(wzd_string_t *ignored, wzd_string_t *command_line, wzd_contex
   i = user->uid;
 
   /* commit to backend */
-  ret = backend_mod_user(mainConfig->backend.name,user->username,user,mod_type);
+  ret = backend_mod_user(mainConfig->backend.filename,old_username,user,mod_type);
 
   /* user has been modified, we have to refresh cache entry */
   user = GetUserByID(i);
@@ -1028,7 +1030,7 @@ int do_site_changegrp(wzd_string_t *ignored, wzd_string_t *command_line, wzd_con
   } /* while (group_name) */
 
   /* commit to backend */
-  backend_mod_user(mainConfig->backend.name,user->username,user,mod_type);
+  backend_mod_user(mainConfig->backend.filename,user->username,user,mod_type);
 
   ret = send_message_with_args(200,context,"User field change successfull");
   return 0;
@@ -1111,7 +1113,7 @@ int do_site_chratio(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   user.ratio = ratio;
 
   /* add it to backend */
-  ret = backend_mod_user(mainConfig->backend.name,str_tochar(username),&user,_USER_RATIO);
+  ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_RATIO);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
@@ -1191,7 +1193,7 @@ int do_site_flags(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context
       return 0;
     }
     /* commit to backend */
-    ret = backend_mod_user(mainConfig->backend.name,str_tochar(username),&user,_USER_FLAGS);
+    ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_FLAGS);
     if (!ret)
       ret = send_message_with_args(200,context,"Flags changed");
     else
@@ -1236,7 +1238,7 @@ int do_site_idle(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
     }
     user->max_idle_time = idletime;
     /* commit to backend */
-    backend_mod_user(mainConfig->backend.name,user->username,user,_USER_IDLE);
+    backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_IDLE);
     snprintf(buffer,1023,"%s","Command ok");
   } else { /* if (*command_line != '\0') */
 
@@ -1268,7 +1270,7 @@ int do_site_tagline(wzd_string_t *ignored, wzd_string_t *command_line, wzd_conte
   if (command_line && strlen(str_tochar(command_line))>0) {
     strncpy(user->tagline,str_tochar(command_line),255);
     /* commit to backend */
-    backend_mod_user(mainConfig->backend.name,user->username,user,_USER_TAGLINE);
+    backend_mod_user(mainConfig->backend.filename,user->username,user,_USER_TAGLINE);
     snprintf(buffer,1023,"%s","Command ok");
   } else { /* if (*command_line != '\0') */
 
@@ -1495,7 +1497,7 @@ int do_site_give(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
     me->credits -= kbytes;
 
   /* add it to backend */
-  ret = backend_mod_user(mainConfig->backend.name,str_tochar(username),&user,_USER_CREDITS);
+  ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_CREDITS);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
@@ -1582,7 +1584,7 @@ int do_site_take(wzd_string_t *ignored, wzd_string_t *command_line, wzd_context_
     user.credits = 0;
 
   /* add it to backend */
-  ret = backend_mod_user(mainConfig->backend.name,str_tochar(username),&user,_USER_CREDITS);
+  ret = backend_mod_user(mainConfig->backend.filename,str_tochar(username),&user,_USER_CREDITS);
 
   if (ret) {
     ret = send_message_with_args(501,context,"Problem changing value");
