@@ -151,7 +151,8 @@ wzd_string_t * str_store(wzd_string_t * str, const char * s)
   length = strlen(s);
   _str_set_min_size(str,length+1);
 
-  wzd_strncpy(str->buffer, s, length);
+  wzd_strncpy(str->buffer, s, length+1);
+  str->length = length;
 
   return str;
 }
@@ -479,10 +480,11 @@ int str_vsprintf(wzd_string_t *str, const char *format, va_list ap)
   /** vsnprintf modifies its last argument on some archs, we have to
    * work on a copy of the va_list
    */
-/*  va_copy(ap2,ap);*/
-  /** (autoconf.info)Function Portability: better use memcpy than va_copy
-   */
+#ifdef va_copy
+  va_copy(ap2,ap);
+#else
   memcpy (&ap2,&ap, sizeof(va_list));
+#endif
 
   result = vsnprintf(str->buffer, str->allocated, format, ap2);
 #ifndef WIN32
@@ -491,7 +493,11 @@ int str_vsprintf(wzd_string_t *str, const char *format, va_list ap)
   {
     _str_set_min_size(str, result+1);
     va_end(ap2);
+#ifdef va_copy
+  va_copy(ap2,ap);
+#else
     memcpy (&ap2,&ap, sizeof(va_list));
+#endif
     result = vsnprintf(str->buffer, str->allocated, format, ap2);
   }
   str->length = result;
@@ -507,13 +513,17 @@ int str_vsprintf(wzd_string_t *str, const char *format, va_list ap)
      }
      _str_set_min_size(str,str->allocated + (str->allocated >> 2) + 20);
      va_end(ap2);
+#ifdef va_copy
+     va_copy(ap2,ap);
+#else
      memcpy (&ap2,&ap, sizeof(va_list));
+#endif
      result = vsnprintf(str->buffer, str->allocated-1, format, ap2);
    }
    str->length = result;
+   str->buffer[str->length] = '\0';
    if ((u32_t)result == str->allocated) {
     _str_set_min_size(str, result+1);
-    str->buffer[str->length] = '\0';
    }
 #endif
 
@@ -595,9 +605,9 @@ int str_append_printf(wzd_string_t *str, const char *format, ...)
     result = vsnprintf(buffer, length-1, format, argptr);
   }
   length = result;
+  buffer[length] = '\0';
   if ((u32_t)result == length) {
    _str_set_min_size(str, result+1);
-   buffer[length] = '\0';
   }
 #endif
 
@@ -777,6 +787,7 @@ static inline void _str_set_min_size(wzd_string_t *str, size_t length)
 
       if (!str->buffer) {
         str->buffer = wzd_malloc(length);
+        str->buffer[0] = '\0';
       } else {
         if ( (ptr = wzd_realloc(str->buffer,length)) ) {
           str->buffer = ptr;
