@@ -76,22 +76,12 @@ int vars_get(const char *varname, void *data, unsigned int datalength, wzd_confi
     snprintf(data,datalength,"%lu",get_bandwidth(NULL,NULL));
     return 0;
   }
-  if (strcmp(varname,"login_pre_ip_check")==0) {
-    snprintf(data,datalength,"%d",config->login_pre_ip_check);
-    return 0;
-  }
   if (strcmp(varname,"loglevel")==0) {
-    char * level;
-    wzd_string_t * str;
+    const char * str;
 
-    str = config_get_string(config->cfg_file, "GLOBAL", "loglevel", NULL);
+    str = config_get_value(config->cfg_file, "GLOBAL", "loglevel");
     if (str) {
-      snprintf(data,datalength,"%s",str_tochar(str));
-      str_deallocate(str);
-      return 0;
-    }
-    if (config->htab && !(chtbl_lookup((CHTBL*)config->htab, "loglevel", (void**)&level))) {
-      snprintf(data,datalength,"%s",level);
+      snprintf(data,datalength,"%s",str);
       return 0;
     }
     snprintf(data,datalength,"%s",loglevel2str(config->loglevel));
@@ -117,7 +107,14 @@ int vars_get(const char *varname, void *data, unsigned int datalength, wzd_confi
     snprintf(data,datalength,"%u",config->pasv_high_range);
     return 0;
   }
-  if (strcasecmp(varname,"port")==0) {
+  if (strcmp(varname,"port")==0) {
+    const char * str;
+
+    str = config_get_value(config->cfg_file, "GLOBAL", "port");
+    if (str) {
+      snprintf(data,datalength,"%s",str);
+      return 0;
+    }
     snprintf(data,datalength,"%u",config->port);
     return 0;
   }
@@ -296,7 +293,7 @@ int vars_user_addip(const char *username, const char *ip, wzd_config_t *config)
   } while (ip);
 
   /* commit to backend */
-  return backend_mod_user(config->backend.name, username, user, _USER_IP);
+  return backend_mod_user(config->backend.filename, username, user, _USER_IP);
 }
 
 int vars_user_delip(const char *username, const char *ip, wzd_config_t *config)
@@ -351,7 +348,7 @@ int vars_user_delip(const char *username, const char *ip, wzd_config_t *config)
   } while (ip);
 
   /* commit to backend */
-  return backend_mod_user(config->backend.name, username, user, _USER_IP);
+  return backend_mod_user(config->backend.filename, username, user, _USER_IP);
 }
 
 int vars_user_set(const char *username, const char *varname, const void *data, unsigned int datalength, wzd_config_t * config)
@@ -474,7 +471,7 @@ int vars_user_set(const char *username, const char *varname, const void *data, u
   }
 
   /* commit to backend */
-  ret = backend_mod_user(config->backend.name, username, user, mod_type);
+  ret = backend_mod_user(config->backend.filename, username, user, mod_type);
 
   return ret;
 }
@@ -537,7 +534,7 @@ int vars_user_new(const char *username, const char *pass, const char *groupname,
   user.leech_slots=0;
 
   /* add it to backend */
-  ret = backend_mod_user(config->backend.name,username,&user,_USER_ALL);
+  ret = backend_mod_user(config->backend.filename,username,&user,_USER_ALL);
 
   return ret;
 }
@@ -654,7 +651,7 @@ int vars_group_set(const char *groupname, const char *varname, const void *data,
   }
 
   /* commit to backend */
-  ret = backend_mod_group(config->backend.name, groupname, group, mod_type);
+  ret = backend_mod_group(config->backend.filename, groupname, group, mod_type);
 
   return ret;
 }
@@ -705,7 +702,7 @@ int vars_group_new(const char *groupname, wzd_config_t * config)
     newgroup.ip_allowed[i][0]='\0';
 
   /* add it to backend */
-  ret = backend_mod_group(config->backend.name,groupname,&newgroup,_GROUP_ALL);
+  ret = backend_mod_group(config->backend.filename,groupname,&newgroup,_GROUP_ALL);
 
   return (ret) ? 1 : 0;
 }
