@@ -71,7 +71,7 @@ int vfs_remove( wzd_vfs_t **vfs_list, const char *vpath )
   {
     next_vfs = current_vfs->next_vfs;
 
-    if ( (strcmp( current_vfs->virtual_dir, vpath) == 0) )
+    if ( (DIRCMP( current_vfs->virtual_dir, vpath) == 0) )
     {
       if (current_vfs == *vfs_list)
       {
@@ -129,7 +129,7 @@ int vfs_add_restricted(wzd_vfs_t ** vfs_list, const char *vpath, const char *pat
   current_vfs = *vfs_list;
   while (current_vfs)
   {
-    if( (strcmp(vpath, current_vfs->virtual_dir)==0) )
+    if( (DIRCMP(vpath, current_vfs->virtual_dir)==0) )
     {
       /* virtual path already set */
       return 2;
@@ -258,7 +258,7 @@ int vfs_replace(wzd_vfs_t *vfs_list, char *buffer, unsigned int maxlen, wzd_cont
     if (DIRNCMP(buffer_vfs,buffer,strlen(buffer_vfs))==0
         &&
         (buffer[strlen(buffer_vfs)] == '/' || /* without this test, vfs will always match before vfs1 */
-      strcmp(buffer_vfs,buffer)==0) ) /* without this test, 'cd vfs' will not match */
+         DIRCMP(buffer_vfs,buffer)==0) ) /* without this test, 'cd vfs' will not match */
     {
       char buf[2*WZD_MAX_PATH];
       /* test perm */
@@ -414,7 +414,7 @@ char *stripdir(char * dir, char *buf, int maxlen)
     {
       if (ldots == 1 || ldots == 2) {
         if (!*in) {
-          if (out-ldots<=dir || *(out-ldots-1) != '/')
+          if (out-ldots<=dir || *(out-ldots-1) != '/') /** \bug XXX FIXME pointers out and dir are NOT in the same buffers */
             ldots = 0;
         }
         while (ldots > 0 && --out > buf)
@@ -500,52 +500,6 @@ printf("Converted to: '%s'\n",path);
   return 0;
 }
 
-int checkabspath(const char *wanted_path, char *path, wzd_context_t *context)
-{
-  char allowed[WZD_MAX_PATH];
-  char cmd[WZD_MAX_PATH];
-
-  {
-    snprintf(allowed,strlen(allowed),"%s/",GetUserByID(context->userid)->rootpath);
-    if (strcmp(allowed,"//")==0) allowed[1]='\0';
-    snprintf(cmd,strlen(cmd),"%s%s",GetUserByID(context->userid)->rootpath,context->currentpath);
-  }
-  if (cmd[strlen(cmd)-1] != '/')
-    strcat(cmd,"/");
-  if (wanted_path) {
-#ifndef _MSC_VER
-    if (wanted_path[0]!='/')
-#else
-    if (wanted_path[0]!='/' && wanted_path[1]!=':')
-#endif
-    {
-      return -1; /* we need absolute path, but it doesn't begin with / */
-    } else {
-      strcpy(cmd,wanted_path);
-    }
-  }
-/*#ifdef DEBUG
-printf("Checking path '%s' (cmd)\nallowed = '%s'\n",cmd,allowed);
-#endif*/
-/*  if (!realpath(cmd,path)) return 1;*/
-  if (!stripdir(cmd,path,WZD_MAX_PATH)) return 1;
-/*#ifdef DEBUG
-printf("Converted to: '%s'\n",path);
-#endif*/
-  if (path[strlen(path)-1] != '/')
-    strcat(path,"/");
-  strcpy(cmd,path);
-  cmd[strlen(allowed)]='\0';
-  if (path[strlen(cmd)-1] != '/')
-    strcat(cmd,"/");
-  /* check if user is allowed to even see the path */
-  if (strncmp(cmd,allowed,strlen(allowed))) return 1;
-  /* in the case of VFS, we need to convert here to a realpath */
-  vfs_replace(mainConfig->vfs,path,WZD_MAX_PATH,context);
-  if (strlen(path)>1 && path[strlen(path)-1] == '/') path[strlen(path)-1]='\0';
-  return 0;
-}
-
 /* FIXME: does not yet support vfs */
 int path_abs2rel(const char *abs, char *rel, int rel_len, wzd_context_t *context)
 {
@@ -598,19 +552,29 @@ int checkpath_new(const char *wanted_path, char *path, wzd_context_t *context)
   fs_filestat_t s;
   struct wzd_file_t * perm_list, * entry;
 
-  user = GetUserByID(context->userid);
-
-  if (!user) return E_USER_IDONTEXIST;
+  WZD_ASSERT(context != NULL);
+  if (context == NULL) return E_USER_IDONTEXIST;
 
   if (!wanted_path) return E_PARAM_NULL;
 
+  if (strlen(context->currentpath) == 0) return E_PARAM_INVALID;
+
+  user = GetUserByID(context->userid);
+
+  if (!user) return E_USER_IDONTEXIST;
   if (strlen(user->rootpath) + strlen(wanted_path) >= WZD_MAX_PATH) return E_PARAM_BIG;
 
   ftppath = malloc(WZD_MAX_PATH+1);
   syspath = malloc(WZD_MAX_PATH+1);
 
-  strncpy(syspath, user->rootpath, WZD_MAX_PATH);
-  sys_offset = strlen(syspath);
+#ifdef WIN32
+  if (strchr(user->flags,FLAG_FULLPATH) )  memset(syspath,0,sizeof(syspath));
+  else
+#endif
+  {
+    strncpy(syspath, user->rootpath, WZD_MAX_PATH);
+    sys_offset = strlen(syspath);
+  }
 
   /* if wanted_path is relative */
   if (wanted_path[0] != '/') {
@@ -653,8 +617,15 @@ int checkpath_new(const char *wanted_path, char *path, wzd_context_t *context)
   ptr_ftppath = ftppath;
   if (*ptr_ftppath == '/')
     ptr_ftppath++;
-  if (syspath[sys_offset-1] != '/')
-    syspath[sys_offset++] = '/';
+
+#ifdef WIN32
+  if (strchr(user->flags,FLAG_FULLPATH) ) sys_offset=0;
+  else
+#endif
+  {
+    if (syspath[sys_offset-1] != '/')
+      memcpy(&syspath[sys_offset++],"/\0",2); /*use either strcat or memcpy with terminating 0 or corruption can occur*/
+  }
 
   while (ptr_ftppath[0] != '\0')
   {
@@ -739,7 +710,7 @@ int checkpath_new(const char *wanted_path, char *path, wzd_context_t *context)
             ptr = buffer_vfs + strlen(syspath);
             /* bingo, vfs */
             /* we overwrite syspath ! */
-            if ( strchr(ptr,'/')==NULL && !strcmp(lpart,ptr) ) { /* not a subdir and same name */
+            if ( strchr(ptr,'/')==NULL && !DIRCMP(lpart,ptr) ) { /* not a subdir and same name */
               strncpy(syspath, vfs->physical_dir, WZD_MAX_PATH);
               sys_offset = strlen(syspath);
               ret = 0;
@@ -780,7 +751,7 @@ int checkpath_new(const char *wanted_path, char *path, wzd_context_t *context)
      */
     if (S_ISDIR(s.mode) || S_ISLNK(s.mode)) {
       if (syspath[sys_offset-1] != '/')
-        syspath[sys_offset++] = '/';
+        memcpy(&syspath[sys_offset++],"/\0",2); /*use either strcat or memcpy with terminating 0 or corruption can occur*/
       if (_checkFileForPerm(syspath,".",RIGHT_CWD,user)) {
         /* no permissions ! */
         free(ftppath);
