@@ -1,9 +1,8 @@
 #include <string.h> /* memset */
 
-#include <wzd_structs.h>
-#include <wzd_action.h>
-
-extern void set_action(wzd_context_t * context, unsigned int token, const char *arg);
+#include <libwzd-core/wzd_structs.h>
+#include <libwzd-core/wzd_string.h>
+#include <libwzd-core/wzd_action.h>
 
 #define C1 0x12345678
 #define C2 0x9abcdef0
@@ -19,13 +18,9 @@ int main(int argc, char *argv[])
 
   memset(&context, 0, sizeof(wzd_context_t));
 
-  set_action(&context, TOK_RETR, buffer);
+  set_action(&context, buffer);
 
-  if ( context.current_action.token != TOK_RETR ) {
-    fprintf(stderr, "Incorrect token\n");
-    return 2;
-  }
-  if ( strlen(context.current_action.arg) != sizeof(context.current_action.arg) ) {
+  if ( str_length(context.current_action.command) != strlen(buffer) ) {
     fprintf(stderr, "Incorrect length\n");
     return 2;
   }
