@@ -99,7 +99,6 @@
 #include <libwzd-core/wzd_utf8.h>
 #include <libwzd-core/wzd_vars.h>
 
-#include "wzd_init.h"
 #include "wzd_ServerThread.h"
 
 #include <libwzd-core/wzd_debug.h>
@@ -133,6 +132,22 @@ typedef struct {
 /************ PUBLIC **************/
 int runMainThread(int argc, char **argv)
 {
+  const char * build_version = WZD_VERSION_STR;
+
+  out_log(LEVEL_FLOOD,"DEBUG Checking library version\n");
+
+  /* check that library was compiled in the same version as executable
+   *
+   * we are trying to avoid the dll hell, especially on windows
+   */
+  if (strcmp(build_version,wzd_get_version_long()) != 0) {
+    out_log(LEVEL_CRITICAL,"FATAL wzdftpd was NOT compiled for this library version\n");
+    out_log(LEVEL_CRITICAL,"FATAL   wzdftpd:     [%s]\n",WZD_VERSION_STR);
+    out_log(LEVEL_CRITICAL,"FATAL   libwzd-core: [%s]\n",wzd_get_version_long());
+
+    exit (-1);
+  }
+
   serverMainThreadProc(0);
 
   return 0;
@@ -161,6 +176,7 @@ static void server_ip_check(fd_set * r_fds, fd_set * w_fds, fd_set * e_fds);
 static void server_ip_select(fd_set * r_fds, fd_set * w_fds, fd_set * e_fds, fd_t * maxfd);
 static void server_ip_free(void * p);
 
+static int check_ip_before_login(wzd_context_t * context);
 
 
 static void context_init(wzd_context_t * context)
@@ -379,10 +395,60 @@ int commit_backend(void)
 {
   /* TODO XXX FIXME flush backend IFF modified ! */
   if (!mainConfig) return 1;
-  backend_commit_changes(mainConfig->backend.name);
+  backend_commit_changes(mainConfig->backend.filename);
   return 0;
 }
 
+static int check_ip_before_login(wzd_context_t * context)
+{
+  char inet_buf[INET6_ADDRSTRLEN]; /* usually 46 */
+  unsigned char * userip;
+  wzd_group_t * loop_group;
+  wzd_user_t * loop_user;
+  gid_t * gid_list;
+  uid_t * uid_list;
+  int i, ret;
+
+  userip = context->hostip;
+#if defined(IPV6_SUPPORT)
+  if (context->family == WZD_INET6) {
+    inet_ntop(AF_INET6,userip,inet_buf,INET6_ADDRSTRLEN);
+  } else
+#endif
+  {
+    inet_ntop(AF_INET,userip,inet_buf,INET_ADDRSTRLEN);
+  }
+
+  /* check for all groups */
+  gid_list = (gid_t*)backend_get_group(-2);
+  if (gid_list) {
+    for (i=0; gid_list[i] != (gid_t)-1; i++) {
+      loop_group = GetGroupByID(gid_list[i]);
+      if (loop_group) {
+        ret = group_ip_inlist(loop_group, inet_buf, context->ident);
+        if (ret) return 0; /* found ! */
+      }
+    }
+    wzd_free(gid_list);
+  }
+
+  /* check for all users */
+  uid_list = (uid_t*)backend_get_user(-2);
+  if (uid_list) {
+    for (i=0; uid_list[i] != (uid_t)-1; i++) {
+      loop_user = GetUserByID(uid_list[i]);
+      if (loop_user) {
+        ret = user_ip_inlist(loop_user, inet_buf, context->ident);
+        if (ret) return 0; /* found ! */
+      }
+    }
+    wzd_free(uid_list);
+  }
+
+
+  return 1;
+}
+
 /*
  * add idents to the correct fd_set
  */
@@ -484,7 +550,7 @@ static int server_add_ident_candidate(fd_t socket_accept_fd)
   context->family = family;
   time (&context->login_time);
 
-  memcpy(context->hostip,userip,16);
+  memcpy(context->hostip,userip,sizeof(context->hostip));
 
   /* check if ident lookups are disabled */
   if (CFG_GET_OPTION(mainConfig,CFG_OPT_DISABLE_IDENT)) {
@@ -601,7 +667,7 @@ static void server_ident_check(fd_set * r_fds, fd_set * w_fds, fd_set * e_fds)
           goto continue_connection;
 #endif
           if (errno == EINPROGRESS) continue;
-          out_log(LEVEL_NORMAL,"error reading ident request %s\n",strerror(errno));
+          out_log(LEVEL_INFO,"error reading ident request %s\n",strerror(errno));
           FD_UNREGISTER(fd_ident,"Ident socket");
           socket_close(fd_ident);
           /* remove ident connection from list and continues with no ident */
@@ -620,7 +686,7 @@ static void server_ident_check(fd_set * r_fds, fd_set * w_fds, fd_set * e_fds)
         }
         ptr++; /* skip ':' */
         while (*ptr && isspace(*ptr)) ptr++;
-        strncpy(context->ident,ptr,MAX_IDENT_LENGTH);
+        context->ident = wzd_strdup(ptr);
         chop(context->ident);
 
 #ifdef WZD_DBG_IDENT
@@ -670,7 +736,7 @@ continue_connection:
           goto continue_connection;
 #endif
           if (errno == EINPROGRESS) continue;
-          out_log(LEVEL_NORMAL,"error sending ident request %s\n",strerror(errno));
+          out_log(LEVEL_INFO,"error sending ident request %s\n",strerror(errno));
           socket_close(fd_ident);
           FD_UNREGISTER(fd_ident,"Ident socket");
           /* remove ident connection from list and continues with no ident */
@@ -740,7 +806,7 @@ static void server_ip_check(fd_set * r_fds, fd_set * w_fds, fd_set * e_fds)
           if (ret != 1 /* global ip rejected */ &&
               ret != 2 /* too many connections */
              )
-            out_log(LEVEL_NORMAL,"could not add connection for connection: %d (errno: %d: %s) :%s:%d\n",
+            out_log(LEVEL_NORMAL,"could not add ident candidate for connection: %d (errno: %d: %s) :%s:%d\n",
                 ret, errno, strerror(errno), __FILE__, __LINE__);
           continue; /* possible cause of error: global ip rejected */
 /*          serverMainThreadExit(-1);*/
@@ -804,6 +870,7 @@ static void server_login_accept(wzd_context_t * context)
       out_log(LEVEL_HIGH,"TLS switch failed (implicit) from client %s\n", inet_buf);
       /* mark context as free */
       context->magic = 0;
+      /** \todo FIXME context is NOT freed */
       return;
     }
     context->connection_flags |= CONNECTION_TLS;
@@ -811,6 +878,19 @@ static void server_login_accept(wzd_context_t * context)
   context->ssl.data_mode = TLS_CLEAR;
 #endif
 
+  if (CFG_GET_OPTION(mainConfig,CFG_OPT_CHECKIP_LOGIN)) {
+    ret = check_ip_before_login(context);
+    if (ret) {
+      close(context->controlfd);
+      FD_UNREGISTER(context->controlfd,"Client socket");
+      out_log(LEVEL_NORMAL,"INFO rejected connection from %s\n",inet_buf);
+      /* mark context as free */
+      context->magic = 0;
+      /** \todo FIXME context is NOT freed */
+      return;
+    }
+  }
+
   /* start new thread */
 
   ret = wzd_thread_attr_init( & thread_attr );
@@ -883,7 +963,7 @@ fprintf(stderr,"Received signal %d\n",signum);
 #endif
 #endif
   /* commit backend changes */
-  ret = backend_commit_changes(mainConfig->backend.name);
+  ret = backend_commit_changes(mainConfig->backend.filename);
   if (ret) {
     out_log(LEVEL_CRITICAL,"Could not commit changes to backend !\n");
   }
@@ -984,6 +1064,12 @@ int server_switch_to_config(wzd_config_t *config)
       if (ipaddress == NULL || strchr(ipaddress,':') == NULL)
         sock4 = socket_make(ipaddress,&port,config->max_threads,WZD_INET4);
 
+      if (sock4 == (fd_t)-1 && sock6 == (fd_t)-1) {
+        out_log(LEVEL_CRITICAL,"FATAL Could not bind to ip %s\n",str_tochar(str_list[i]));
+        str_deallocate_array(str_list);
+        return -1;
+      }
+
       if (sock6 != (fd_t)-1) {
         server_ip = wzd_malloc(sizeof(*server_ip));
         server_ip->dynamic = 0;
@@ -993,6 +1079,7 @@ int server_switch_to_config(wzd_config_t *config)
         if (list_ins_next(&server_ip_list, NULL, server_ip)) {
           out_log(LEVEL_CRITICAL,"FATAL Could not register server ip %s\n",ptr_to_data);
           server_ip_free(server_ip);
+          str_deallocate_array(str_list);
           return -1;
         }
         {
@@ -1000,6 +1087,7 @@ int server_switch_to_config(wzd_config_t *config)
 
           if (setsockopt(sock6, SOL_SOCKET, SO_KEEPALIVE, (char *) &one, sizeof(int)) < 0) {
             out_log(LEVEL_CRITICAL,"setsockopt(SO_KEEPALIVE");
+            str_deallocate_array(str_list);
             return -1;
           }
         }
@@ -1013,6 +1101,7 @@ int server_switch_to_config(wzd_config_t *config)
         if (list_ins_next(&server_ip_list, NULL, server_ip)) {
           out_log(LEVEL_CRITICAL,"FATAL Could not register server ip %s\n",ptr_to_data);
           server_ip_free(server_ip);
+          str_deallocate_array(str_list);
           return -1;
         }
         {
@@ -1020,6 +1109,7 @@ int server_switch_to_config(wzd_config_t *config)
 
           if (setsockopt(sock4, SOL_SOCKET, SO_KEEPALIVE, (char *) &one, sizeof(int)) < 0) {
             out_log(LEVEL_CRITICAL,"setsockopt(SO_KEEPALIVE");
+            str_deallocate_array(str_list);
             return -1;
           }
         }
@@ -1027,49 +1117,6 @@ int server_switch_to_config(wzd_config_t *config)
 
     }
     str_deallocate_array(str_list);
-  } else { /* code for compatibility with old format, to be removed */
-    sock6 = socket_make(NULL,&mainConfig->port,config->max_threads,WZD_INET6);
-    sock4 = socket_make(NULL,&mainConfig->port,config->max_threads,WZD_INET4);
-    if (sock6 != (fd_t)-1) {
-      server_ip = wzd_malloc(sizeof(*server_ip));
-      server_ip->dynamic = 0;
-      server_ip->ip = NULL;
-      server_ip->sock = sock6;
-      FD_REGISTER(sock6,"Server listening socket");
-      if (list_ins_next(&server_ip_list, NULL, server_ip)) {
-        out_log(LEVEL_CRITICAL,"FATAL Could not register server ip %d\n",mainConfig->port);
-        server_ip_free(server_ip);
-        return -1;
-      }
-      {
-        int one=1;
-
-        if (setsockopt(sock6, SOL_SOCKET, SO_KEEPALIVE, (char *) &one, sizeof(int)) < 0) {
-          out_log(LEVEL_CRITICAL,"setsockopt(SO_KEEPALIVE");
-          return -1;
-        }
-      }
-    }
-    if (sock4 != (fd_t)-1) {
-      server_ip = wzd_malloc(sizeof(*server_ip));
-      server_ip->dynamic = 0;
-      server_ip->ip = NULL;
-      server_ip->sock = sock4;
-      FD_REGISTER(sock4,"Server listening socket");
-      if (list_ins_next(&server_ip_list, NULL, server_ip)) {
-        out_log(LEVEL_CRITICAL,"FATAL Could not register server ip %d\n",mainConfig->port);
-        server_ip_free(server_ip);
-        return -1;
-      }
-      {
-        int one=1;
-
-        if (setsockopt(sock4, SOL_SOCKET, SO_KEEPALIVE, (char *) &one, sizeof(int)) < 0) {
-          out_log(LEVEL_CRITICAL,"setsockopt(SO_KEEPALIVE");
-          return -1;
-        }
-      }
-    }
   }
 
 /** \bug XXX FIXME polling with select on named pipe seems to fail ... */
@@ -1153,11 +1200,11 @@ int server_switch_to_config(wzd_config_t *config)
 
 
   /* if no backend available, we must bail out - otherwise there would be no login/pass ! */
-  if (!mainConfig->backend.name) {
+  if (!mainConfig->backend.filename) {
     out_log(LEVEL_CRITICAL,"I have no backend ! I must die, otherwise you will have no login/pass !!\n");
     return -1;
   }
-  ret = backend_init(config->backend.name,0 /* max users */,0 /* max groups */);
+  ret = backend_init(config->backend.filename,0 /* max users */,0 /* max groups */);
   /* if no backend available, we must bail out - otherwise there would be no login/pass ! */
   if (ret || config->backend.handle == NULL) {
     out_log(LEVEL_CRITICAL,"I have no backend ! I must die, otherwise you will have no login/pass !!\n");
@@ -1280,13 +1327,10 @@ int server_switch_to_config(wzd_config_t *config)
 void serverMainThreadProc(void *arg)
 {
   int ret;
+  unsigned long max_wait_time;
   fd_set r_fds, w_fds, e_fds;
   fd_t maxfd;
   struct timeval tv;
-#if defined(_MSC_VER) || (defined(__CYGWIN__) && defined(WINSOCK_SUPPORT))
-  WSADATA wsaData;
-  int nCode;
-#endif
 
 #ifndef WIN32
   /* catch broken pipe ! */
@@ -1347,15 +1391,6 @@ void serverMainThreadProc(void *arg)
 #endif
 #endif /* _WIN32 */
 
-#if defined(_MSC_VER)
-  /* Start Winsock up */
-  if ((nCode = WSAStartup(MAKEWORD(2, 0), &wsaData)) != 0) {
-    out_log(LEVEL_CRITICAL,"Error initializing winsock2 %s:%d\n",
-      __FILE__, __LINE__);
-    exit(-1);
-  }
-#endif
-
   /******** init shm *******/
   /* do this _before_ loading config, config can use it ! */
   vars_shm_init();
@@ -1436,6 +1471,12 @@ void serverMainThreadProc(void *arg)
 
   out_log(LEVEL_INFO,"Process %d ok\n",getpid());
 
+  /* get value for server tick */
+  max_wait_time = config_get_integer(mainConfig->cfg_file, "GLOBAL", "server tick", &ret);
+  if (ret != CF_OK) {
+    max_wait_time = DEFAULT_SERVER_TICK;
+  }
+
   /* sets start time, for uptime */
   time(&mainConfig->server_start);
 
@@ -1454,7 +1495,7 @@ void serverMainThreadProc(void *arg)
     FD_ZERO(&w_fds);
     FD_ZERO(&e_fds);
 
-    tv.tv_sec = HARD_REACTION_TIME; tv.tv_usec = 0;
+    tv.tv_sec = max_wait_time; tv.tv_usec = 0;
     maxfd = 0;
     server_ip_select(&r_fds, &w_fds, &e_fds, &maxfd);
     server_ident_select(&r_fds, &w_fds, &e_fds, &maxfd);
@@ -1501,7 +1542,7 @@ void serverMainThreadProc(void *arg)
 
 
   /* commit backend changes */
-  ret = backend_commit_changes(mainConfig->backend.name);
+  ret = backend_commit_changes(mainConfig->backend.filename);
   if (ret) {
     out_log(LEVEL_CRITICAL,"Could not commit changes to backend !\n");
   } else
@@ -1515,10 +1556,6 @@ static void free_config(wzd_config_t * config)
 /*  limiter_free(mainConfig->limiter_ul);
   limiter_free(mainConfig->limiter_dl);*/
 
-  ip_list_free(mainConfig->login_pre_ip_allowed);
-
-  ip_list_free(mainConfig->login_pre_ip_denied);
-
   if (mainConfig->xferlog_fd >= 0)
     xferlog_close(mainConfig->xferlog_fd);
 
@@ -1532,26 +1569,6 @@ static void free_config(wzd_config_t * config)
   wzd_free(mainConfig->pid_file);
   wzd_free(mainConfig->dir_message);
 
-  wzd_free(mainConfig->site_config.file_ginfo);
-  mainConfig->site_config.file_ginfo = NULL;
-  wzd_free(mainConfig->site_config.file_group);
-  mainConfig->site_config.file_group = NULL;
-  wzd_free(mainConfig->site_config.file_groups);
-  mainConfig->site_config.file_groups = NULL;
-  wzd_free(mainConfig->site_config.file_help);
-  mainConfig->site_config.file_help = NULL;
-  wzd_free(mainConfig->site_config.file_rules);
-  mainConfig->site_config.file_rules = NULL;
-  wzd_free(mainConfig->site_config.file_swho);
-  mainConfig->site_config.file_swho = NULL;
-  wzd_free(mainConfig->site_config.file_user);
-  mainConfig->site_config.file_user = NULL;
-  wzd_free(mainConfig->site_config.file_users);
-  mainConfig->site_config.file_users = NULL;
-  wzd_free(mainConfig->site_config.file_vfs);
-  mainConfig->site_config.file_vfs = NULL;
-  wzd_free(mainConfig->site_config.file_who);
-  mainConfig->site_config.file_who = NULL;
   wzd_free(mainConfig);
 }
 
@@ -1636,7 +1653,7 @@ void serverMainThreadExit(int retcode)
   hook_free(&mainConfig->hook);
   hook_free_protocols();
   module_free(&mainConfig->module);
-  backend_close(mainConfig->backend.name);
+  backend_close(mainConfig->backend.filename);
   cronjob_free(&mainConfig->crontab);
   section_free(&mainConfig->section_list);
   vfs_free(&mainConfig->vfs);
@@ -1678,7 +1695,8 @@ void serverMainThreadExit(int retcode)
   signal(SIGSEGV,SIG_DFL);
 #endif
 
-  wzd_mutex_unlock(server_mutex);
+  wzd_mutex_unlock(end_mutex);
+  wzd_mutex_destroy(end_mutex);
 
   exit (retcode);
 }
