@@ -84,7 +84,6 @@
 #include <libwzd-core/wzd_libmain.h>
 #include <libwzd-core/wzd_utf8.h>
 
-#include "wzd_init.h"
 #include "wzd_opts.h"
 #include "wzd_ServerThread.h"
 
@@ -162,13 +161,6 @@ void display_usage(void)
 #endif /* HAVE_GETOPT_LONG */
 }
 
-static wzd_config_t * load_config_file(const char *name, wzd_config_t ** config)
-{
-  *config = readConfigFile(name);
-
-  return *config;
-}
-
 
 int main_parse_args(int argc, char **argv)
 {
@@ -300,9 +292,22 @@ int main(int argc, char **argv)
   pid_t forkresult;
   wzd_config_t * config;
   wzd_configfile_t * cf;
+#if defined(WIN32)
+  WSADATA wsaData;
+  int nCode;
+#endif
 
   wzd_debug_init();
 
+#if defined(WIN32)
+  /* Start Winsock up */
+  if ((nCode = WSAStartup(MAKEWORD(2, 0), &wsaData)) != 0) {
+    out_err(LEVEL_CRITICAL,"Error initializing winsock2 %s:%d\n",
+      __FILE__, __LINE__);
+    exit(-1);
+  }
+#endif
+
 #if 0
   fprintf(stderr,"--------------------------------------\n");
   fprintf(stderr,"\n");
@@ -426,9 +431,7 @@ int main(int argc, char **argv)
       }
     }
     config_free(cf);
-/*    if (!ret) break;*/ /** \todo FIXME enable this line when config is really loaded */
-    /* old config loader */
-    if (load_config_file(config_files[i],&config)) break;
+    if (!ret) break;
   }
   if (!config) {
     fprintf(stderr,"FATAL: No valid config file found, aborting !\n");
@@ -471,30 +474,8 @@ int main(int argc, char **argv)
   }
 
 
-  mainConfig = wzd_malloc(sizeof(wzd_config_t));
-
-#if DEBUG
-#if 0
-  /** \todo XXX this is a test ! */
-  cf = config_new();
-  ret = config_set_value(cf, "[GLOBAL]", "key", "value");
-  ret = config_load_from_file (cf, config->config_filename, CF_FILE_MERGE_MULTIPLE);
-  fprintf(stderr,"  config_filename(%p,%s,0) -> %d\n",
-      cf, config->config_filename,ret);
-  {
-    wzd_string_t * str;
-    str = config_to_data(cf, NULL);
-
-    if (str) printf("%s\n",str_tochar(str));
-    str_deallocate(str);
-  }
-  config_free(cf);
-  /** \todo XXX end of test ! */
-#endif /* 0 */
-#endif
-
+  mainConfig = config;
   setlib_mainConfig(mainConfig);
-  memcpy(mainConfig,config,sizeof(wzd_config_t));
 
 #ifndef WIN32
   if (CFG_GET_OPTION(mainConfig,CFG_OPT_USE_SYSLOG)) {
@@ -506,18 +487,20 @@ int main(int argc, char **argv)
       log_set_syslog(i,1);
   }
 #endif
-  ret = log_open(mainConfig->logfilename,mainConfig->logfilemode);
-  if (ret < 0) {
-    /* stderr is not closed here, even in release mode */
-    fprintf(stderr,"FATAL: Could not open log file %s\n",mainConfig->logfilename);
-    return 1;
-  }
-  /** \todo this should be removed (as well as log_get() function) and replace
-   * with a proper init code
-   */
-  for (i=0; i<MAX_LOG_CHANNELS; i++) {
-    if (log_get(i) == -1)
-      log_set(i,ret);
+  if (mainConfig->logfilename != NULL) {
+    ret = log_open(mainConfig->logfilename,mainConfig->logfilemode);
+    if (ret < 0) {
+      /* stderr is not closed here, even in release mode */
+      fprintf(stderr,"FATAL: Could not open log file %s\n",mainConfig->logfilename);
+      return 1;
+    }
+    /** \todo this should be removed (as well as log_get() function) and replace
+     * with a proper init code
+     */
+    for (i=0; i<MAX_LOG_CHANNELS; i++) {
+      if (log_get(i) == -1)
+        log_set(i,ret);
+    }
   }
 
 #if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
